/*
 iA6/iA6B Receiver - RSSI PWM output with 4 sample averaging
 Version: 1.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 24/11/2015
 Last Modified: 3/11/2016
 
 Monitors the RSSI byte on the iA6 receiver and output it as 50Hz PWM (1ms to 2ms as the high period) 
 to be used on an OSD like MW OSD running MinimOSD hardware.
 For MW OSD set RSSI Min to 126 and RSSI Max to 255, a bit of adjust may be needed. 
 Connect ATtiny RSSI PWM PB4 to MinimOSD's ATmega328 pin A3 (PC3).
 Power the ATtiny with 5V.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL and divide clock by 1
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0xe1:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 */
 
// ATtiny25/45/85 Pin map
//
//                  +-\/-+
//      Reset PB5  1|o   |8  VCC
//            PB3  2|    |7  PB2 Clock
//   RSSI PWM PB4  3|    |6  PB1 
//            GND  4|    |5  PB0 Data
//                  +----+

#define F_CPU 16000000 // 16 MHz

#define rssiPWMout PB4

#define RSSI_AVG_NO 4

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <math.h>
#include "setup.c"

uint8_t rssiByte = 1;
uint8_t signalLostCounter = 0;
uint8_t rssiByteAverage[RSSI_AVG_NO];
uint8_t rssiAvgCounter = 0;

int main(void) {
	setup();
	
	// Enable USI with negative clock edge
	USICR = (1<<USIWM0) | (1<<USICS1) | (1<<USICS0); 
	
	uint8_t usiResyncCounter = 0;
	while(1) {
		while (!(USISR & (1<<USIOIF))); // Wait until new serial data received
		USISR |= (1<<USIOIF); // Clear received flag
		
		// Wait for FE and 00 sequence, then RSSI byte is next
		if (USIBR == 0xFE) {
			while (!(USISR & (1<<USIOIF)));
			USISR |= (1<<USIOIF);
			
			if (USIBR == 0x00) {
				while (!(USISR & (1<<USIOIF)));
				USISR |= (1<<USIOIF);
				
				// RSSI byte
				if (USIBR == 255) {
					rssiByteAverage[rssiAvgCounter] = 1; // Minimum of 1 required for shortest high period
					rssiAvgCounter++;
				}
				else if (USIBR <= 84) {
					rssiByteAverage[rssiAvgCounter] = 255 - (USIBR * 3); // The higher error rate, the lower the RSSI signal goes
					rssiAvgCounter++;
				}
				
				// Calculate RSSI Average
				if (rssiAvgCounter >= RSSI_AVG_NO) {
					uint16_t rssiAvgCalc = 0;
					for (uint8_t x = 0; x < RSSI_AVG_NO; x++) {
						rssiAvgCalc += rssiByteAverage[x];
					}
					
					rssiByte = (rssiAvgCalc / RSSI_AVG_NO);
				}
				
				signalLostCounter = 0; // Reset signal lost counter
			}
		}
		else if (USIBR == 0xFF) { // If receiving 8+ FFs in a row then we're near the end and can re-sync the USI to the proper clocks (if we ever started out of sync)
			usiResyncCounter++;
			if (usiResyncCounter >= 8) {
				_delay_us(500);
				USISR = (1<<USIOIF); // Clear received flag and clock counter bits
			}
		}
		else {
			usiResyncCounter = 0;
		}
		
		// If we don't detect RSSI byte for a while then we assume we've lost the signal
		if (signalLostCounter >= 100) {
			rssiByte = 1;
			signalLostCounter = 100;
		}
	}
}

// Pulse start for 1ms
ISR(TIMER1_COMPA_vect) {
	PORTB |= (1<<rssiPWMout);
	signalLostCounter++;
}

// Turn off timer 1 and turn on timer 0 for pulse width (1ms max generation to give us 1-2ms high period)
ISR(TIMER1_OVF_vect) {
	TCCR1 = 0;
	
	OCR0A = rssiByte; // Update OCR0A from RSSI byte
	
	TCNT0 = 0;
	TCCR0B = (1<<CS01) | (1<<CS00);
}

// Turn off timer 0 and turn on timer 1 for ~50Hz PWM generation
ISR(TIMER0_COMPA_vect) {
	TCCR0B = 0;
	PORTB &= ~(1<<rssiPWMout);
	
	TCNT1 = 0;
	TCCR1 = (1<<CTC1) | (1<<PWM1A) | (1<<CS13) | (1<<CS12);	
}
